c
c                          demo2.f
c
c     A demo program demonstrating how to produce a flyby sequence 
c     of a 3D particle distribution. The program generates 181 GIF 
c     files numbered sequentially. Use any conversion program to 
c     generate a movie file (in MPEG or any other format)
c
c        Andrey Kravtsov ( New Mexico State University, 1998 ) 
c
c     Copyright (c) 1998  Andrey V. Kravtsov. This program can be distributed
c     and used freely for personal, educational or research purposes.  
c     Use for any other purposes requires special arrangements.  
c     
c     Questions and/or comments may be sent to akravtso@nmsu.edu
c
c
      implicit none
c
      integer npmax
      parameter ( npmax = 300000 ) 
      integer ind(npmax)
      real xp(npmax) , yp(npmax) , zp(npmax) , dp(npmax)
c
      integer np , ic1 , id, icount
      real psu(11) , aexpn, F(8,3)
      integer P
      real SIGN, CONTRA, BRIGHT
      real Phi, Theta
      character*15 fname
      logical ContinueSequence
c
      integer  InitGIFDevice
      external InitGIFDevice
c
c.... read particles
c
      write(*,*) 'reading particles...'
      open ( 10 , file = 'particles.dat' , form='unformatted', 
     &            status='old')
      read(10) np
      write(*,*) 'np = ', np
      read(10) aexpn
      write(*,*) 'aexpn = ', aexpn
      do ic1 = 1 , np 
        read(10) xp(ic1), yp(ic1), zp(ic1), dp(ic1)
      enddo
      close ( 10 ) 
c
c.... sort particles
c
      write(*,*) 'sorting particles...'
      call SortParticles ( dp , np , ind )
c
c.... set up a view point
c
      Phi   = 0.0
      Theta = 120.0 
      psu(1)  = 0.0    ! x-coordinate (3D) of the box center
      psu(2)  = 0.0    ! y-coordinate (3D) of the box center
      psu(3)  = 0.0    ! z-coordinate (3D) of the box center
      psu(4)  = 0.5    ! scale of the plot for zoom in/out
      psu(9)  = 0.0    ! x-coordinate (3D) of the viewing point
      psu(10) = 0.0    ! y-coordinate (3D) of the viewing point
      psu(11) = 2.5    ! z-coordinate (3D) of the viewing point
c      
c.... set up a palette
c
      P      = 4
      SIGN   = 1.0
      CONTRA = 0.56
      BRIGHT = 0.5 
c
c.... Initialize frame count 
c     
      icount = 0 
      ContinueSequence = .true.
c
      DO WHILE ( ContinueSequence )
c        
        Theta  = Theta + 2.0 
        icount = icount + 1
        psu(3) = - 2.5* (icount-1)/179.
c
c....   Initialize GIF frame
c
        call GetName ( icount , fname ) 
        call InitGIFDevice ( fname )
        write(*,*) 'generating frame # ',icount
c
        call SetViewAngle ( Phi, Theta, psu(5), psu(6), psu(7), psu(8) )
c
c....   plot particles
c
        call pgbbuf ()
        call pgeras ()      
c
        call PlotParticlesPoints ( xp , yp , zp , dp , ind , np , 
     &                             psu , P , SIGN , CONTRA , BRIGHT )
c
        call InitFrame ( F )
        call DrawFrame ( F , psu )
        call pgebuf
c
        if ( icount .gt. 180 ) ContinueSequence = .false. 
c
      ENDDO
c
c.... End PGPLOT 
c
      call pgend
c
      STOP
      END
c
c     ---------------------------------------------
      subroutine SortParticles ( dp , npart , ind )
c     ---------------------------------------------
c
c     Sort particles according to their local density
c     ind(np) has index of particle with largest local density
c
c     input  : npart     - number of particles to sort
c              dp(npart) - density (or any other attribute) to use in sorting
c     output : ind(npart)- particle indices in sorted order
c     
c     uses: indexx subroutine from Numerical Recipes
c
      implicit none
c
      integer npart
      real dp(npart) 
      integer ind(npart)
c
      call indexx ( npart , dp , ind )
c
      return
      end
c
c     ---------------------------------------------------------------
      subroutine PlotParticlesPoints ( xp, yp, zp, dp, ind, npart, 
     &                                 psu, P, SIGN, CONTRA, BRIGHT )
c     ---------------------------------------------------------------
c  
c     Plots input particles coloring them according to the input densities
c
c
      implicit none
c
      integer npart
      integer ind(npart)
      real xp(npart), yp(npart), zp(npart), dp(npart)
      real psu(11)
      integer P
      real CONTRA , BRIGHT , SIGN
c
      integer CI1 , CI2 , ic1 , ip , icolor
      real dmin , dmax , dlmin , dlmax, xpr, ypr
c
      dmin = 1.e7
      dmax = -dmin
      do ic1 = 1 , npart
        if ( dp(ic1) .lt. dmin ) dmin = dp(ic1)
        if ( dp(ic1) .gt. dmax ) dmax = dp(ic1)
      enddo
c
      dlmax = log(dmax)
      if ( dmin .gt. 0.0 ) then 
        dlmin = log(dmin)
      else
        dlmin = -3
      endif
c
      CI1 = 16 
      CI2 = 86
      call PGSCIR ( CI1 , CI2 )         
c
      call Palett ( P , SIGN*CONTRA , BRIGHT )
c
      call PGSITF ( 0 ) 
      call PGSCLP ( 0 )  ! no clipping
c
      do ic1 = 1 , npart
        ip = ind(ic1)
        call OnScreen ( xp(ip) , yp(ip) , zp(ip) , psu , xpr , ypr )
        if ( dp(ip) .gt. 0.0 ) then 
          if ( dp(ip).gt.dmax ) then 
            icolor = 1
          else 
            icolor = int((CI2-CI1)*(log(dp(ip))-dlmin)/(dlmax-dlmin)) 
     &                + CI1
          endif
        else 
          icolor = CI1
        endif
        call pgsci ( icolor )
        call pgpt1 ( xpr , ypr , 1 ) 
      enddo
c
      return
      end
c
c     -----------------------------
      SUBROUTINE indexx(n,arr,indx)
c     -----------------------------
c     (C) Copr. 1986-92 Numerical Recipes Software .-35?421.1-9.
c     ----------------------------------------------------------
      INTEGER n,indx(n),M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,indxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      REAL a
      do 11 j=1,n
        indx(j)=j
11    continue
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 13 j=l+1,ir
          indxt=indx(j)
          a=arr(indxt)
          do 12 i=j-1,1,-1
            if(arr(indx(i)).le.a)goto 2
            indx(i+1)=indx(i)
12        continue
          i=0
2         indx(i+1)=indxt
13      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        itemp=indx(k)
        indx(k)=indx(l+1)
        indx(l+1)=itemp
        if(arr(indx(l+1)).gt.arr(indx(ir)))then
          itemp=indx(l+1)
          indx(l+1)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l)).gt.arr(indx(ir)))then
          itemp=indx(l)
          indx(l)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l+1)).gt.arr(indx(l)))then
          itemp=indx(l+1)
          indx(l+1)=indx(l)
          indx(l)=itemp
        endif
        i=l+1
        j=ir
        indxt=indx(l)
        a=arr(indxt)
3       continue
          i=i+1
        if(arr(indx(i)).lt.a)goto 3
4       continue
          j=j-1
        if(arr(indx(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=indx(i)
        indx(i)=indx(j)
        indx(j)=itemp
        goto 3
5       indx(l)=indx(j)
        indx(j)=indxt
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in indexx'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END

